@@ -19,29 +19,39 @@
 # chaining as the spec requires.
 
 import copy
+from typing import TYPE_CHECKING, Dict, List, Optional, TypeVar, Union
 
 from appium.webdriver.mobilecommand import MobileCommand as Command
 
+if TYPE_CHECKING:
+    from appium.webdriver.webdriver import WebDriver
+    from appium.webdriver.webelement import WebElement
+    from appium.webdriver.common.touch_action import TouchAction
 
-class MultiAction(object):
-    def __init__(self, driver, element=None):
+T = TypeVar('T', bound='MultiAction')
+
+
+class MultiAction:
+    def __init__(self, driver: 'WebDriver', element: Optional['WebElement'] = None) -> None:
         self._driver = driver
         self._element = element
-        self._touch_actions = []
+        self._touch_actions: List['TouchAction'] = []
 
-    def add(self, *touch_actions):
+    def add(self, *touch_actions: 'TouchAction') -> None:
         """Add TouchAction objects to the MultiAction, to be performed later.
 
         Args:
-            touch_actions (`TouchAction`): one or more TouchAction objects describing a chain of actions to be performed by one finger
+            touch_actions: one or more TouchAction objects describing a chain of actions to be performed by one finger
 
         Usage:
-            a1 = TouchAction(driver)
-            a1.press(el1).move_to(el2).release()
-            a2 = TouchAction(driver)
-            a2.press(el2).move_to(el1).release()
+            | a1 = TouchAction(driver)
+            | a1.press(el1).move_to(el2).release()
+            | a2 = TouchAction(driver)
+            | a2.press(el2).move_to(el1).release()
+            | MultiAction(driver).add(a1, a2)
 
-            MultiAction(driver).add(a1, a2)
+        Returns:
+            `MultiAction`: Self instance
         """
         for touch_action in touch_actions:
             if self._touch_actions is None:
@@ -49,16 +59,18 @@ class MultiAction(object):
 
             self._touch_actions.append(copy.copy(touch_action))
 
-    def perform(self):
+    def perform(self: T) -> T:
         """Perform the actions stored in the object.
 
         Usage:
-            a1 = TouchAction(driver)
-            a1.press(el1).move_to(el2).release()
-            a2 = TouchAction(driver)
-            a2.press(el2).move_to(el1).release()
+            | a1 = TouchAction(driver)
+            | a1.press(el1).move_to(el2).release()
+            | a2 = TouchAction(driver)
+            | a2.press(el2).move_to(el1).release()
+            | MultiAction(driver).add(a1, a2).perform()
 
-            MultiAction(driver).add(a1, a2).perform()
+        Returns:
+            `MultiAction`: Self instance
         """
         self._driver.execute(Command.MULTI_ACTION, self.json_wire_gestures)
 
@@ -68,7 +80,7 @@ class MultiAction(object):
         return self
 
     @property
-    def json_wire_gestures(self):
+    def json_wire_gestures(self) -> Dict[str, Union[List, str]]:
         actions = []
         for action in self._touch_actions:
             actions.append(action.json_wire_gestures)
