@@ -24,25 +24,34 @@
 # pylint: disable=no-self-use
 
 import copy
+from typing import TYPE_CHECKING, Dict, List, Optional, TypeVar, Union
 
 from appium.webdriver.mobilecommand import MobileCommand as Command
 
+if TYPE_CHECKING:
+    from appium.webdriver.webelement import WebElement
+    from appium.webdriver.webdriver import WebDriver
 
-class TouchAction(object):
-    def __init__(self, driver=None):
+T = TypeVar('T', bound='TouchAction')
+
+
+class TouchAction:
+
+    def __init__(self, driver: Optional['WebDriver'] = None):
         self._driver = driver
-        self._actions = []
+        self._actions: List = []
 
-    def tap(self, element=None, x=None, y=None, count=1):
+    def tap(self: T, element: Optional['WebElement'] = None, x: Optional[int]
+            = None, y: Optional[int] = None, count: int = 1) -> T:
         """Perform a tap action on the element
 
         Args:
-            element (`appium.webdriver.webelement.WebElement`): the element to tap
-            x (:obj:`int`, optional): x coordinate to tap, relative to the top left corner of the element.
-            y (:obj:`int`, optional): y coordinate. If y is used, x must also be set, and vice versa
+            element: the element to tap
+            x : x coordinate to tap, relative to the top left corner of the element.
+            y : y coordinate. If y is used, x must also be set, and vice versa
 
         Returns:
-            `TouchAction`: self instance
+            `TouchAction`: Self instance
         """
         opts = self._get_opts(element, x, y)
         opts['count'] = count
@@ -50,47 +59,49 @@ class TouchAction(object):
 
         return self
 
-    def press(self, el=None, x=None, y=None, pressure=None):
+    def press(self: T, el: Optional['WebElement'] = None, x: Optional[int] = None,
+              y: Optional[int] = None, pressure: Optional[float] = None) -> T:
         """Begin a chain with a press down action at a particular element or point
 
         Args:
-            el (:obj:`appium.webdriver.webelement.WebElement`, optional): the element to press
-            x (:obj:`int`, optional): x coordiate to press. If y is used, x must also be set
-            y (:obj:`int`, optional): y coordiate to press. If x is used, y must also be set
-            pressure (:obj:`float`, optional): [iOS Only] press as force touch. Read the description of `force` property on Apple's UITouch class
+            el: the element to press
+            x: x coordiate to press. If y is used, x must also be set
+            y: y coordiate to press. If x is used, y must also be set
+            pressure: [iOS Only] press as force touch. Read the description of `force` property on Apple's UITouch class
                                 (https://developer.apple.com/documentation/uikit/uitouch?language=objc) for more details on possible value ranges.
 
         Returns:
-            `TouchAction`: self instance
+            `TouchAction`: Self instance
         """
         self._add_action('press', self._get_opts(el, x, y, pressure=pressure))
 
         return self
 
-    def long_press(self, el=None, x=None, y=None, duration=1000):
+    def long_press(self: T, el: Optional['WebElement'] = None, x: Optional[int]
+                   = None, y: Optional[int] = None, duration: int = 1000) -> T:
         """Begin a chain with a press down that lasts `duration` milliseconds
 
         Args:
-            el (:obj:`appium.webdriver.webelement.WebElement`, optional): the element to press
-            x (:obj:`int`, optional): x coordiate to press. If y is used, x must also be set
-            y (:obj:`int`, optional): y coordiate to press. If x is used, y must also be set
-            duration (:obj:`int`, optional): Duration to press
+            el: the element to press
+            x: x coordiate to press. If y is used, x must also be set
+            y: y coordiate to press. If x is used, y must also be set
+            duration: Duration to press
 
         Returns:
-            `TouchAction`: self instance
+            `TouchAction`: Self instance
         """
         self._add_action('longPress', self._get_opts(el, x, y, duration))
 
         return self
 
-    def wait(self, ms=0):
+    def wait(self: T, ms: int = 0) -> T:
         """Pause for `ms` milliseconds.
 
         Args:
-            ms (int): The time to pause
+            ms: The time to pause
 
         Returns:
-            `TouchAction`: self instance
+            `TouchAction`: Self instance
         """
         if ms is None:
             ms = 0
@@ -101,37 +112,39 @@ class TouchAction(object):
 
         return self
 
-    def move_to(self, el=None, x=None, y=None):
+    def move_to(self: T, el: Optional['WebElement'] = None, x: Optional[int] = None, y: Optional[int] = None) -> T:
         """Move the pointer from the previous point to the element or point specified
 
         Args:
-            el (:obj:`appium.webdriver.webelement.WebElement`, optional): the element to be moved to
-            x (:obj:`int`, optional): x coordiate to be moved to. If y is used, x must also be set
-            y (:obj:`int`, optional): y coordiate to be moved to. If x is used, y must also be set
+            el: the element to be moved to
+            x: x coordiate to be moved to. If y is used, x must also be set
+            y: y coordiate to be moved to. If x is used, y must also be set
 
         Returns:
-            `TouchAction`: self instance
+            `TouchAction`: Self instance
         """
         self._add_action('moveTo', self._get_opts(el, x, y))
 
         return self
 
-    def release(self):
+    def release(self: T) -> T:
         """End the action by lifting the pointer off the screen
 
         Returns:
-            `TouchAction`: self instance
+            `TouchAction`: Self instance
         """
         self._add_action('release', {})
 
         return self
 
-    def perform(self):
+    def perform(self: T) -> T:
         """Perform the action by sending the commands to the server to be operated upon
 
         Returns:
-            `TouchAction`: self instance
+            `TouchAction`: Self instance
         """
+        if self._driver is None:
+            raise ValueError('Set driver to constructor as a argument when to create the instance.')
         params = {'actions': self._actions}
         self._driver.execute(Command.TOUCH_ACTION, params)
 
@@ -141,23 +154,24 @@ class TouchAction(object):
         return self
 
     @property
-    def json_wire_gestures(self):
+    def json_wire_gestures(self) -> List[Dict]:
         gestures = []
         for action in self._actions:
             gestures.append(copy.deepcopy(action))
         return gestures
 
-    def _add_action(self, action, options):
+    def _add_action(self, action: str, options: Dict) -> None:
         gesture = {
             'action': action,
             'options': options,
         }
         self._actions.append(gesture)
 
-    def _get_opts(self, element, x, y, duration=None, pressure=None):
+    def _get_opts(self, el: Optional['WebElement'] = None, x: Optional[int] = None, y: Optional[int] = None,
+                  duration: Optional[int] = None, pressure: Optional[float] = None) -> Dict[str, Union[int, float]]:
         opts = {}
-        if element is not None:
-            opts['element'] = element.id
+        if el is not None:
+            opts['element'] = el.id
 
         # it makes no sense to have x but no y, or vice versa.
         if x is not None and y is not None:
