@@ -12,28 +12,37 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING, List, Optional, Tuple, TypeVar, Union
+
 from selenium import webdriver
 
 from appium.webdriver.common.multi_action import MultiAction
 from appium.webdriver.common.touch_action import TouchAction
+from appium.webdriver.webelement import WebElement
+
+if TYPE_CHECKING:
+    # noinspection PyUnresolvedReferences
+    from appium.webdriver.webdriver import WebDriver
+
+T = TypeVar('T', bound=Union['WebDriver', 'ActionHelpers'])
 
 
 class ActionHelpers(webdriver.Remote):
 
-    def scroll(self, origin_el, destination_el, duration=None):
+    def scroll(self: T, origin_el: WebElement, destination_el: WebElement, duration: Optional[int] = None) -> T:
         """Scrolls from one element to another
 
         Args:
-            originalEl (`appium.webdriver.webelement.WebElement`): the element from which to being scrolling
-            destinationEl (`appium.webdriver.webelement.WebElement`): the element to scroll to
-            duration (int): a duration after pressing originalEl and move the element to destinationEl.
+            origin_el: the element from which to being scrolling
+            destination_el: the element to scroll to
+            duration: a duration after pressing originalEl and move the element to destinationEl.
                 Default is 600 ms for W3C spec. Zero for MJSONWP.
 
         Usage:
             driver.scroll(el1, el2)
 
         Returns:
-            `appium.webdriver.webelement.WebElement`
+            Union['WebDriver', 'ActionHelpers']: Self instance
         """
 
         # XCUITest x W3C spec has no duration by default in server side
@@ -47,34 +56,34 @@ class ActionHelpers(webdriver.Remote):
             action.press(origin_el).wait(duration).move_to(destination_el).release().perform()
         return self
 
-    def drag_and_drop(self, origin_el, destination_el):
+    def drag_and_drop(self: T, origin_el: WebElement, destination_el: WebElement) -> T:
         """Drag the origin element to the destination element
 
         Args:
-            originEl (`appium.webdriver.webelement.WebElement`): the element to drag
-            destinationEl (`appium.webdriver.webelement.WebElement`): the element to drag to
+            origin_el: the element to drag
+            destination_el: the element to drag to
 
         Returns:
-            `appium.webdriver.webelement.WebElement`
+            Union['WebDriver', 'ActionHelpers']: Self instance
         """
         action = TouchAction(self)
         action.long_press(origin_el).move_to(destination_el).release().perform()
         return self
 
-    def tap(self, positions, duration=None):
+    def tap(self: T, positions: List[Tuple[int, int]], duration: Optional[int] = None) -> T:
         """Taps on an particular place with up to five fingers, holding for a
         certain time
 
         Args:
-            positions (:obj:`list` of :obj:`tuple`): an array of tuples representing the x/y coordinates of
+            positions: an array of tuples representing the x/y coordinates of
                 the fingers to tap. Length can be up to five.
-            duration (:obj:`int`, optional): length of time to tap, in ms
+            duration: length of time to tap, in ms
 
         Usage:
             driver.tap([(100, 20), (100, 60), (100, 100)], 500)
 
         Returns:
-            `appium.webdriver.webelement.WebElement`
+            Union['WebDriver', 'ActionHelpers']: Self instance
         """
         if len(positions) == 1:
             action = TouchAction(self)
@@ -100,21 +109,21 @@ class ActionHelpers(webdriver.Remote):
             ma.perform()
         return self
 
-    def swipe(self, start_x, start_y, end_x, end_y, duration=None):
+    def swipe(self: T, start_x: int, start_y: int, end_x: int, end_y: int, duration: int = 0) -> T:
         """Swipe from one point to another point, for an optional duration.
 
         Args:
-            start_x (int): x-coordinate at which to start
-            start_y (int): y-coordinate at which to start
-            end_x (int): x-coordinate at which to stop
-            end_y (int): y-coordinate at which to stop
-            duration (:obj:`int`, optional): time to take the swipe, in ms.
+            start_x: x-coordinate at which to start
+            start_y: y-coordinate at which to start
+            end_x: x-coordinate at which to stop
+            end_y: y-coordinate at which to stop
+            duration: time to take the swipe, in ms.
 
         Usage:
             driver.swipe(100, 100, 100, 400)
 
         Returns:
-            `appium.webdriver.webelement.WebElement`
+            Union['WebDriver', 'ActionHelpers']: Self instance
         """
         # `swipe` is something like press-wait-move_to-release, which the server
         # will translate into the correct action
@@ -127,20 +136,20 @@ class ActionHelpers(webdriver.Remote):
         action.perform()
         return self
 
-    def flick(self, start_x, start_y, end_x, end_y):
+    def flick(self: T, start_x: int, start_y: int, end_x: int, end_y: int) -> T:
         """Flick from one point to another point.
 
         Args:
-            start_x (int): x-coordinate at which to start
-            start_y (int): y-coordinate at which to start
-            end_x (int): x-coordinate at which to stop
-            end_y (int): y-coordinate at which to stop
+            start_x: x-coordinate at which to start
+            start_y: y-coordinate at which to start
+            end_x: x-coordinate at which to stop
+            end_y: y-coordinate at which to stop
 
         Usage:
             driver.flick(100, 100, 100, 400)
 
         Returns:
-            `appium.webdriver.webelement.WebElement`
+            Union['WebDriver', 'ActionHelpers']: Self instance
         """
         action = TouchAction(self)
         action \
