@@ -12,26 +12,37 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING, TypeVar, Union
+
 from selenium import webdriver
 
 from appium.webdriver.mobilecommand import MobileCommand as Command
 
+if TYPE_CHECKING:
+    # noinspection PyUnresolvedReferences
+    from appium.webdriver.webdriver import WebDriver
+
+T = TypeVar('T', bound=Union['WebDriver', 'Display'])
+
 
 class Display(webdriver.Remote):
 
-    def get_display_density(self):
+    def get_display_density(self: T) -> int:
         """Get the display density, Android only
 
         Returns:
-            int: The display density of the Android device(dpi)
+            The display density of the Android device(dpi)
 
         Usage:
             self.driver.get_display_density()
+
+        Return:
+            int: The display density
         """
         return self.execute(Command.GET_DISPLAY_DENSITY)['value']
 
     # pylint: disable=protected-access
-
-    def _addCommands(self):
+    # noinspection PyProtectedMember
+    def _addCommands(self) -> None:
         self.command_executor._commands[Command.GET_DISPLAY_DENSITY] = \
             ('GET', '/session/$sessionId/appium/device/display_density')
