@@ -12,20 +12,28 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING, Any, Dict, TypeVar, Union
+
 from selenium import webdriver
 
 from ..mobilecommand import MobileCommand as Command
 
+if TYPE_CHECKING:
+    # noinspection PyUnresolvedReferences
+    from appium.webdriver.webdriver import WebDriver
+
+T = TypeVar('T', bound=Union['WebDriver', 'Applications'])
+
 
 class Applications(webdriver.Remote):
-    def background_app(self, seconds):
+    def background_app(self: T, seconds: int) -> T:
         """Puts the application in the background on the device for a certain duration.
 
         Args:
-            seconds (int): the duration for the application to remain in the background
+            seconds: the duration for the application to remain in the background
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'Applications']: Self instance
         """
         data = {
             'seconds': seconds,
@@ -33,25 +41,25 @@ class Applications(webdriver.Remote):
         self.execute(Command.BACKGROUND, data)
         return self
 
-    def is_app_installed(self, bundle_id):
+    def is_app_installed(self: T, bundle_id: str) -> bool:
         """Checks whether the application specified by `bundle_id` is installed on the device.
 
         Args:
-            bundle_id (str): the id of the application to query
+            bundle_id: the id of the application to query
 
         Returns:
-            bool: `True` if app is installed
+            `True` if app is installed
         """
         data = {
             'bundleId': bundle_id,
         }
         return self.execute(Command.IS_APP_INSTALLED, data)['value']
 
-    def install_app(self, app_path, **options):
+    def install_app(self: T, app_path: str, **options: Any) -> T:
         """Install the application found at `app_path` on the device.
 
         Args:
-            app_path (str): the local or remote path to the application to install
+            app_path: the local or remote path to the application to install
 
         Keyword Args:
             replace (bool): [Android only] whether to reinstall/upgrade the package if it is
@@ -65,9 +73,9 @@ class Applications(webdriver.Remote):
                 on Android 6+ after the installation completes. False by default
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'Applications']: Self instance
         """
-        data = {
+        data: Dict[str, Any] = {
             'appPath': app_path,
         }
         if options:
@@ -75,11 +83,11 @@ class Applications(webdriver.Remote):
         self.execute(Command.INSTALL_APP, data)
         return self
 
-    def remove_app(self, app_id, **options):
+    def remove_app(self: T, app_id: str, **options: Any) -> T:
         """Remove the specified application from the device.
 
         Args:
-            app_id (str): the application id to be removed
+            app_id: the application id to be removed
 
         Keyword Args:
             keepData (bool): [Android only] whether to keep application data and caches after it is uninstalled.
@@ -88,9 +96,9 @@ class Applications(webdriver.Remote):
                 20000ms by default.
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'Applications']: Self instance
         """
-        data = {
+        data: Dict[str, Any] = {
             'appId': app_id,
         }
         if options:
@@ -98,54 +106,54 @@ class Applications(webdriver.Remote):
         self.execute(Command.REMOVE_APP, data)
         return self
 
-    def launch_app(self):
+    def launch_app(self: T) -> T:
         """Start on the device the application specified in the desired capabilities.
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'Applications']: Self instance
         """
         self.execute(Command.LAUNCH_APP)
         return self
 
-    def close_app(self):
+    def close_app(self: T) -> T:
         """Stop the running application, specified in the desired capabilities, on
         the device.
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'Applications']: Self instance
         """
         self.execute(Command.CLOSE_APP)
         return self
 
-    def terminate_app(self, app_id, **options):
+    def terminate_app(self: T, app_id: str, **options: Any) -> bool:
         """Terminates the application if it is running.
 
         Args:
-            app_id (str): the application id to be terminates
+            app_id: the application id to be terminates
 
         Keyword Args:
             `timeout` (int): [Android only] how much time to wait for the uninstall to complete.
                 500ms by default.
 
         Returns:
-            bool: True if the app has been successfully terminated
+            True if the app has been successfully terminated
         """
-        data = {
+        data: Dict[str, Any] = {
             'appId': app_id,
         }
         if options:
             data.update({'options': options})
         return self.execute(Command.TERMINATE_APP, data)['value']
 
-    def activate_app(self, app_id):
+    def activate_app(self: T, app_id: str) -> T:
         """Activates the application if it is not running
         or is running in the background.
 
         Args:
-            app_id (str): the application id to be activated
+            app_id: the application id to be activated
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'Applications']: Self instance
         """
         data = {
             'appId': app_id,
@@ -153,11 +161,11 @@ class Applications(webdriver.Remote):
         self.execute(Command.ACTIVATE_APP, data)
         return self
 
-    def query_app_state(self, app_id):
+    def query_app_state(self: T, app_id: str) -> int:
         """Queries the state of the application.
 
         Args:
-            app_id (str): the application id to be queried
+            app_id: the application id to be queried
 
         Returns:
             One of possible application state constants. See ApplicationState
@@ -168,13 +176,16 @@ class Applications(webdriver.Remote):
         }
         return self.execute(Command.QUERY_APP_STATE, data)['value']
 
-    def app_strings(self, language=None, string_file=None):
+    def app_strings(self: T, language: str = None, string_file: str = None) -> Dict[str, str]:
         """Returns the application strings from the device for the specified
         language.
 
         Args:
-            language (str): strings language code
-            string_file (str): the name of the string file to query
+            language: strings language code
+            string_file: the name of the string file to query
+
+        Returns:
+            The key is string id and the value is the content.
         """
         data = {}
         if language is not None:
@@ -183,15 +194,18 @@ class Applications(webdriver.Remote):
             data['stringFile'] = string_file
         return self.execute(Command.GET_APP_STRINGS, data)['value']
 
-    def reset(self):
+    def reset(self: T) -> T:
         """Resets the current application on the device.
+
+        Returns:
+            Union['WebDriver', 'Applications']: Self instance
         """
         self.execute(Command.RESET)
         return self
 
     # pylint: disable=protected-access
-
-    def _addCommands(self):
+    # noinspection PyProtectedMember
+    def _addCommands(self) -> None:
         self.command_executor._commands[Command.BACKGROUND] = \
             ('POST', '/session/$sessionId/appium/app/background')
         self.command_executor._commands[Command.IS_APP_INSTALLED] = \
