@@ -12,12 +12,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING, Any, Dict, TypeVar, Union
+
 from selenium import webdriver
 
+if TYPE_CHECKING:
+    # noinspection PyUnresolvedReferences
+    from appium.webdriver.webdriver import WebDriver
+
+T = TypeVar('T', bound=Union['WebDriver', 'ExecuteMobileCommand'])
+
 
 class ExecuteMobileCommand(webdriver.Remote):
 
-    def press_button(self, button_name):
+    def press_button(self: T, button_name: str) -> T:
         """Sends a physical button name to the device to simulate the user pressing.
 
         iOS only.
@@ -25,10 +33,10 @@ class ExecuteMobileCommand(webdriver.Remote):
         https://github.com/appium/WebDriverAgent/blob/master/WebDriverAgentLib/Categories/XCUIDevice%2BFBHelpers.h
 
         Args:
-            button_name (str): the button name to be sent to the device
+            button_name: the button name to be sent to the device
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'ExecuteMobileCommand']: Self instance
 
         """
         data = {
@@ -38,7 +46,7 @@ class ExecuteMobileCommand(webdriver.Remote):
         return self
 
     @property
-    def battery_info(self):
+    def battery_info(self: T) -> Dict[str, Any]:
         """Retrieves battery information for the device under test.
 
         Returns:
