@@ -12,25 +12,37 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING, Dict, Optional, TypeVar, Union
+
 from selenium import webdriver
 
 from ..mobilecommand import MobileCommand as Command
 
+if TYPE_CHECKING:
+    # noinspection PyUnresolvedReferences
+    from appium.webdriver.webdriver import WebDriver
+
+T = TypeVar('T', bound=Union['WebDriver', 'Keyboard'])
+
 
 class Keyboard(webdriver.Remote):
 
-    def hide_keyboard(self, key_name=None, key=None, strategy=None):
+    def hide_keyboard(self: T, key_name: Optional[str] = None, key: Optional[str]
+                      = None, strategy: Optional[str] = None) -> T:
         """Hides the software keyboard on the device.
 
         In iOS, use `key_name` to press
         a particular key, or `strategy`. In Android, no parameters are used.
 
         Args:
-            key_name (:obj:`str`, optional): key to press
-            key (:obj:`str`, optional):
-            strategy (:obj:`str`, optional): strategy for closing the keyboard (e.g., `tapOutside`)
+            key_name: key to press
+            key:
+            strategy: strategy for closing the keyboard (e.g., `tapOutside`)
+
+        Returns:
+            Union['WebDriver', 'Keyboard']: Self instance
         """
-        data = {}
+        data: Dict[str, Optional[str]] = {}
         if key_name is not None:
             data['keyName'] = key_name
         elif key is not None:
@@ -41,26 +53,26 @@ class Keyboard(webdriver.Remote):
         self.execute(Command.HIDE_KEYBOARD, data)
         return self
 
-    def is_keyboard_shown(self):
+    def is_keyboard_shown(self: T) -> bool:
         """Attempts to detect whether a software keyboard is present
 
         Returns:
-            bool: `True` if keyboard is shown
+            `True` if keyboard is shown
         """
         return self.execute(Command.IS_KEYBOARD_SHOWN)['value']
 
-    def keyevent(self, keycode, metastate=None):
+    def keyevent(self: T, keycode: int, metastate: Optional[int] = None) -> T:
         """Sends a keycode to the device.
 
         Android only.
         Possible keycodes can be found in http://developer.android.com/reference/android/view/KeyEvent.html.
 
         Args:
-            keycode (int): the keycode to be sent to the device
-            metastate (:obj:`int`, optional): meta information about the keycode being sent
+            keycode: the keycode to be sent to the device
+            metastate: meta information about the keycode being sent
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'Keyboard']: Self instance
         """
         data = {
             'keycode': keycode,
@@ -70,18 +82,19 @@ class Keyboard(webdriver.Remote):
         self.execute(Command.KEY_EVENT, data)
         return self
 
-    def press_keycode(self, keycode, metastate=None, flags=None):
+    def press_keycode(self: T, keycode: int, metastate: Optional[int] = None, flags: Optional[int] = None) -> T:
         """Sends a keycode to the device.
 
-        Android only. Possible keycodes can be found in http://developer.android.com/reference/android/view/KeyEvent.html.
+        Android only. Possible keycodes can be found
+        in http://developer.android.com/reference/android/view/KeyEvent.html.
 
         Args:
-            keycode (int): the keycode to be sent to the device
-            metastate (:obj:`int`, optional): meta information about the keycode being sent
-            flags (:obj:`int`, optional): the set of key event flags
+            keycode: the keycode to be sent to the device
+            metastate: meta information about the keycode being sent
+            flags: the set of key event flags
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'Keyboard']: Self instance
         """
         data = {
             'keycode': keycode,
@@ -93,18 +106,19 @@ class Keyboard(webdriver.Remote):
         self.execute(Command.PRESS_KEYCODE, data)
         return self
 
-    def long_press_keycode(self, keycode, metastate=None, flags=None):
+    def long_press_keycode(self: T, keycode: int, metastate: Optional[int] = None, flags: Optional[int] = None) -> T:
         """Sends a long press of keycode to the device.
 
-        Android only. Possible keycodes can be found in http://developer.android.com/reference/android/view/KeyEvent.html.
+        Android only. Possible keycodes can be found in
+        http://developer.android.com/reference/android/view/KeyEvent.html.
 
         Args:
-            keycode (int): the keycode to be sent to the device
-            metastate (:obj:`int`, optional): meta information about the keycode being sent
-            flags (:obj:`int`, optional): the set of key event flags
+            keycode: the keycode to be sent to the device
+            metastate: meta information about the keycode being sent
+            flags: the set of key event flags
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'Keyboard']: Self instance
         """
         data = {
             'keycode': keycode
@@ -117,8 +131,8 @@ class Keyboard(webdriver.Remote):
         return self
 
     # pylint: disable=protected-access
-
-    def _addCommands(self):
+    # noinspection PyProtectedMember
+    def _addCommands(self) -> None:
         self.command_executor._commands[Command.HIDE_KEYBOARD] = \
             ('POST', '/session/$sessionId/appium/device/hide_keyboard')
         self.command_executor._commands[Command.IS_KEYBOARD_SHOWN] = \
