@@ -13,54 +13,63 @@
 # limitations under the License.
 
 import base64
+from typing import TYPE_CHECKING, Optional, TypeVar, Union
 
 from selenium import webdriver
 from selenium.common.exceptions import InvalidArgumentException
 
 from ..mobilecommand import MobileCommand as Command
 
+if TYPE_CHECKING:
+    # noinspection PyUnresolvedReferences
+    from appium.webdriver.webdriver import WebDriver
+
+T = TypeVar('T', bound=Union['WebDriver', 'RemoteFS'])
+
 
 class RemoteFS(webdriver.Remote):
-    def pull_file(self, path):
+    def pull_file(self: T, path: str) -> str:
         """Retrieves the file at `path`.
 
         Args:
-            path (str): the path to the file on the device
+            path: the path to the file on the device
 
         Returns:
-            bytes: The file's contents as base64.
+            The file's contents encoded as Base64.
         """
         data = {
             'path': path,
         }
         return self.execute(Command.PULL_FILE, data)['value']
 
-    def pull_folder(self, path):
+    def pull_folder(self: T, path: str) -> str:
         """Retrieves a folder at `path`.
 
         Args:
-            path (str): the path to the folder on the device
+            path: the path to the folder on the device
 
         Returns:
-            bytes: The folder's contents zipped and encoded as Base64.
+            The folder's contents zipped and encoded as Base64.
         """
         data = {
             'path': path,
         }
         return self.execute(Command.PULL_FOLDER, data)['value']
 
-    def push_file(self, destination_path, base64data=None, source_path=None):
+    def push_file(self: T, destination_path: str,
+                  base64data: Optional[str] = None, source_path: Optional[str] = None) -> T:
         """Puts the data from the file at `source_path`, encoded as Base64, in the file specified as `path`.
 
         Specify either `base64data` or `source_path`, if both specified default to `source_path`
 
         Args:
-            destination_path (str): the location on the device/simulator where the local file contents should be saved
-            base64data (:obj:`bytes`, optional): file contents, encoded as Base64, to be written to the file on the device/simulator
-            source_path (:obj:`str`, optional): local file path for the file to be loaded on device
+            destination_path: the location on the device/simulator where the local file contents should be saved
+            base64data: file contents, encoded as Base64, to be written
+            to the file on the device/simulator
+            source_path: local file path for the file to be loaded on device
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'RemoteFS']: Self instance
         """
         if source_path is None and base64data is None:
             raise InvalidArgumentException('Must either pass base64 data or a local file path')
@@ -68,11 +77,11 @@ class RemoteFS(webdriver.Remote):
         if source_path is not None:
             try:
                 with open(source_path, 'rb') as f:
-                    data = f.read()
-            except IOError:
-                message = 'source_path {} could not be found. Are you sure the file exists?'.format(source_path)
-                raise InvalidArgumentException(message)
-            base64data = base64.b64encode(data).decode('utf-8')
+                    file_data = f.read()
+            except IOError as e:
+                message = f'source_path "{source_path}" could not be found. Are you sure the file exists?'
+                raise InvalidArgumentException(message) from e
+            base64data = base64.b64encode(file_data).decode('utf-8')
 
         data = {
             'path': destination_path,
@@ -82,8 +91,8 @@ class RemoteFS(webdriver.Remote):
         return self
 
     # pylint: disable=protected-access
-
-    def _addCommands(self):
+    # noinspection PyProtectedMember
+    def _addCommands(self) -> None:
         self.command_executor._commands[Command.PULL_FILE] = \
             ('POST', '/session/$sessionId/appium/device/pull_file')
         self.command_executor._commands[Command.PULL_FOLDER] = \
