@@ -12,30 +12,38 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING, Any, Dict, TypeVar, Union
+
 from selenium import webdriver
 
 from ..mobilecommand import MobileCommand as Command
 
+if TYPE_CHECKING:
+    # noinspection PyUnresolvedReferences
+    from appium.webdriver.webdriver import WebDriver
+
+T = TypeVar('T', bound=Union['WebDriver', 'Settings'])
+
 
 class Settings(webdriver.Remote):
-    def get_settings(self):
+    def get_settings(self: T) -> Dict[str, Any]:
         """Returns the appium server Settings for the current session.
 
         Do not get Settings confused with Desired Capabilities, they are
         separate concepts. See https://github.com/appium/appium/blob/master/docs/en/advanced-concepts/settings.md
 
         Returns:
-            dict: Current settings
+            Current settings
         """
         return self.execute(Command.GET_SETTINGS, {})['value']
 
-    def update_settings(self, settings):
+    def update_settings(self: T, settings: Dict[str, Any]) -> T:
         """Set settings for the current session.
 
         For more on settings, see: https://github.com/appium/appium/blob/master/docs/en/advanced-concepts/settings.md
 
         Args:
-            settings (dict): dictionary of settings to apply to the current test session
+            settings: dictionary of settings to apply to the current test session
         """
         data = {"settings": settings}
 
@@ -43,8 +51,8 @@ class Settings(webdriver.Remote):
         return self
 
     # pylint: disable=protected-access
-
-    def _addCommands(self):
+    # noinspection PyProtectedMember
+    def _addCommands(self) -> None:
         self.command_executor._commands[Command.GET_SETTINGS] = \
             ('GET', '/session/$sessionId/appium/settings')
         self.command_executor._commands[Command.UPDATE_SETTINGS] = \
