@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import Dict, List, Optional, TypeVar, Union
+
 from selenium.webdriver.common.by import By
 from selenium.webdriver.common.utils import keys_to_typing
 from selenium.webdriver.remote.command import Command as RemoteCommand
@@ -19,15 +21,11 @@ from selenium.webdriver.remote.command import Command as RemoteCommand
 from .extensions.search_context import AppiumWebElementSearchContext
 from .mobilecommand import MobileCommand as Command
 
-# Python 3 imports
-try:
-    str = basestring
-except NameError:
-    pass
+T = TypeVar('T', bound='WebElement')
 
 
 class WebElement(AppiumWebElementSearchContext):
-    def get_attribute(self, name):
+    def get_attribute(self, name: str) -> Optional[Union[str, Dict]]:
         """Gets the given attribute or property of the element.
 
         Override for Appium
@@ -43,14 +41,15 @@ class WebElement(AppiumWebElementSearchContext):
         is returned.
 
         Args:
-            name (str): Name of the attribute/property to retrieve.
+            name: Name of the attribute/property to retrieve.
 
         Usage:
             # Check if the "active" CSS class is applied to an element.
+
             is_active = "active" in target_element.get_attribute("class")
 
         Returns:
-            str: The given attribute or property of the element
+            The given attribute or property of the element
         """
 
         resp = self._execute(RemoteCommand.GET_ELEMENT_ATTRIBUTE, {'name': name})
@@ -62,25 +61,23 @@ class WebElement(AppiumWebElementSearchContext):
         if isinstance(attributeValue, dict):
             return attributeValue
 
+        # Convert to str along to the spec
         if not isinstance(attributeValue, str):
-            try:
-                attributeValue = unicode(attributeValue)
-            except NameError:
-                attributeValue = str(attributeValue)
+            attributeValue = str(attributeValue)
 
         if name != 'value' and attributeValue.lower() in ('true', 'false'):
             return attributeValue.lower()
 
         return attributeValue
 
-    def is_displayed(self):
+    def is_displayed(self) -> bool:
         """Whether the element is visible to a user.
 
         Override for Appium
         """
         return self._execute(RemoteCommand.IS_ELEMENT_DISPLAYED)['value']
 
-    def find_element(self, by=By.ID, value=None):
+    def find_element(self, by: str = By.ID, value: Union[str, Dict] = None) -> T:
         """Find an element given a By strategy and locator
 
         Override for Appium
@@ -88,8 +85,8 @@ class WebElement(AppiumWebElementSearchContext):
         Prefer the find_element_by_* methods when possible.
 
         Args:
-            by (:obj:`str`, optional): The strategy
-            value (:obj:`str`, optional): The locator
+            by: The strategy
+            value: The locator
 
         Usage:
             element = element.find_element(By.ID, 'foo')
@@ -114,7 +111,7 @@ class WebElement(AppiumWebElementSearchContext):
         return self._execute(RemoteCommand.FIND_CHILD_ELEMENT,
                              {"using": by, "value": value})['value']
 
-    def find_elements(self, by=By.ID, value=None):
+    def find_elements(self, by: str = By.ID, value: Union[str, Dict] = None) -> List[T]:
         """Find elements given a By strategy and locator
 
         Override for Appium
@@ -122,8 +119,8 @@ class WebElement(AppiumWebElementSearchContext):
         Prefer the find_elements_by_* methods when possible.
 
         Args:
-            by (:obj:`str`, optional): The strategy
-            value (:obj:`str`, optional): The locator
+            by: The strategy
+            value: The locator
 
         Usage:
             element = element.find_elements(By.CLASS_NAME, 'foo')
@@ -148,7 +145,7 @@ class WebElement(AppiumWebElementSearchContext):
         return self._execute(RemoteCommand.FIND_CHILD_ELEMENTS,
                              {"using": by, "value": value})['value']
 
-    def clear(self):
+    def clear(self) -> T:
         """Clears text.
 
         Override for Appium
@@ -160,14 +157,14 @@ class WebElement(AppiumWebElementSearchContext):
         self._execute(Command.CLEAR, data)
         return self
 
-    def set_text(self, keys=''):
+    def set_text(self, keys: str = '') -> T:
         """Sends text to the element.
 
         Previous text is removed.
         Android only.
 
         Args:
-            keys (str): the text to be sent to the element.
+            keys: the text to be sent to the element.
 
         Usage:
             element.set_text('some text')
@@ -183,24 +180,24 @@ class WebElement(AppiumWebElementSearchContext):
         return self
 
     @property
-    def location_in_view(self):
+    def location_in_view(self) -> Dict[str, int]:
         """Gets the location of an element relative to the view.
 
         Usage:
-            location = element.location_in_view
-            x = location['x']
-            y = location['y']
+            | location = element.location_in_view
+            | x = location['x']
+            | y = location['y']
 
         Returns:
             dict: The location of an element relative to the view
         """
         return self._execute(Command.LOCATION_IN_VIEW)['value']
 
-    def set_value(self, value):
+    def set_value(self, value: str) -> T:
         """Set the value on this element in the application
 
         Args:
-            value (str): The value to be set
+            value: The value to be set
 
         Returns:
             `appium.webdriver.webelement.WebElement`
@@ -213,11 +210,11 @@ class WebElement(AppiumWebElementSearchContext):
         return self
 
     # Override
-    def send_keys(self, *value):
+    def send_keys(self, *value: str) -> T:
         """Simulates typing into the element.
 
         Args:
-            value (str): A string for typing.
+            value: A string for typing.
 
         Returns:
             `appium.webdriver.webelement.WebElement`
